#ifndef CHECKERS
#define CHECKERS
/****************************************************************************/
/*                                                                          */
/*                            Checker Declarations                          */
/*                            --------------------                          */
/*                                                                          */
/* Written by: John Thornley, Computer Science Dept., Caltech.              */
/* Last modified:Tuesday July 5, 1994.                                      */
/*                                                                          */
/* Purpose: Declarations of types and functions for the game Checkers.      */
/*                                                                          */
/****************************************************************************/

#include <stdio.h>
#include "boolean.h"

/*-------------------------------------------------------------------------*/
/*                                   TYPES                                 */
/*-------------------------------------------------------------------------*/

extern int max_depth;
extern int p_mob;

#define NUM_SQUARES 33
#define MAX_HOPS 12
#define MAX_MOVES 42
#define NON_EXISTENT_SQUARE 0
#define MAN_VALUE 2
#define KING_VALUE 3
#define MAX_PIECES 12
#define POSITION_VALUE  2
#define BRIDGE_VALUE 24
#define MOBILITY_VALUE 1
#define length num_moves
#define list mlist

typedef int square;

#define NUM_DIRECTIONS 4

typedef enum {up_left, up_right, down_left, down_right} direction;

#define NUM_COLORS 2

typedef enum {black, white} color;

#define NUM_OCCUPANTS 6
#define NUM_PIECE_TYPES 4

/* order of occupant types affects position weight array.  */
typedef enum {black_man, black_king, white_man, white_king, empty,
		off_board} occupant;

typedef occupant piece;

typedef struct {
  int      val;
  color    to_move;
  occupant board[NUM_SQUARES];
} game;

typedef struct {
  square hop_from,
	 hop_over,
	 hop_to;
} hop;

typedef struct {
  boolean is_jump;
  square move_from,      /* no jump case */
	 move_to;        /* no jump case */
  int val,
      num_hops;          /* jump case    */
  hop hops[MAX_HOPS];    /* jump case    */
} move;

typedef struct {
  int length;
  move list[MAX_MOVES];
} moves;

typedef enum {black_win, white_win, unfinished} game_status;

typedef struct {
  game_status status;
  int win_depth,  /* if status!=black or white win, value is random */
      black_pieces,
      white_pieces,
      mobility;   /* (black - white) mobility */
} score;

/*----------------------------------------------------------------------------*/
/*                        VALIDITY CHECKING FUNCTIONS                         */
/*----------------------------------------------------------------------------*/

boolean valid_color(color the_color);
/* Return: the_color is a valid color. */

boolean valid_occupant(occupant the_occupant);
/* Return: the_occupant is a valid occupant. */

boolean valid_game(game the_game);
/* Return: the_game is a valid game. */

boolean valid_move(game the_game, move the_move);
/* Precondition: valid_game(game).             */
/* Return: the_move is a valid move from game. */

boolean valid_score(score the_score);
/* Return: the_score is a valid score. */

void generate_jumps(game *the_game, moves *the_moves);
void generate_non_jumps(game *the_game, moves *move_list);
/* Precondition: valid_game(game).              */
/* Return: list of valid moves for the position */

boolean generate_jumps_helper(occupant board[33], square the_square,
	  move *jump_so_far,int this_hop, direction forward_right,
	  direction forward_left, direction backward_right,
	  direction backward_left, piece my_king, moves *move_list,
	  occupant orig_board[33]);

boolean move_in_generated_list(move the_move, moves move_list);
/* Precondition: Generated move list                    */
/* Return: True if move is in the list, false otherwise */

/*----------------------------------------------------------------------------*/
/*                          GAME AND MOVE FUNCTIONS                           */
/*----------------------------------------------------------------------------*/

color other_color(color the_color);
/* Precondition: valid_color(the_color).   */
/* Return: The other color than the_color. */

void start_game(game *the_game);
/* Postcondition: *the_game is the starting game. */

void apply_move(game *the_game, move the_move);
/* Precondition: valid_game(*the_game) and valid_move(*the_game, the_move). */
/* Postcondition: *the_game'out is *the_game'in after the_move is applied.  */

void undo_one_jump(occupant *board, hop the_hop, occupant *orig_board);
/* Un-jumps the piece after leaf is evaluated or branch exhausted           */

score static_evaluation(game *the_game, int *prior_mobility, int depth);
/* Precondition: valid_game(the_game).    */
/* Return: Static evaluation of the_game. */

int compare_scores(score left, score right);
/* Precondition: valid_score(left) and valid_score(right). */
/* Return: -1, if left is worse than right for black.      */
/*          0, if left and right are equal.                */
/*         +1, if left is better than right for black.     */


score white_win_game(int depth);
score black_win_game(int depth);

score test_win(game position);

/*----------------------------------------------------------------------------*/
/*                           INPUT/OUTPUT FUNCTIONS                           */
/*----------------------------------------------------------------------------*/

void print_color(color the_color);
/* Precondition: valid_color(the_color).                   */
/* Postcondition: the_color is written in ASCII to stdout. */

void print_game(game the_game);
/* Precondition: valid_game(the_game).                    */
/* Postcondition: the_game is written in ASCII to stdout. */

void print_move(move the_move);
/* Precondition: Exists some g for which valid_move(g, the_move). */
/* Postcondition: the_move is written in ASCII to stdout.         */

void print_score(score the_score);
/* Precondition: valid_score(the_score).                   */
/* Postcondition: the_score is written in ASCII to stdout. */

boolean scan_color(color *the_color);
/* Return: *the_color is successfully read in ASCII from stdin.           */
/* Postcondition: If successful, *the_color is the color read from stdin. */

boolean scan_game(game *the_game);
/* Return: *the_game is successfully read in ASCII from stdin.          */
/* Postcondition: If successful, *the_game is the game read from stdin. */

boolean scan_move(move *the_move, moves move_list);
/* Return: *the_move is successfully read in ASCII from stdin.          */
/* Postcondition: If successful. *the_move is the move read from stdin. */


char print_occupant(int position, game the_game);
/* returns char to print-change name. */

/****************************************************************************/
#endif /* CHECKERS */
